@@ -14,7 +14,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1335  USA
 
 *******************************************************/
 
@@ -75,6 +75,7 @@ datasink_t datasink_compress = {
 	&compress_open,
 	&compress_write,
 	&compress_close,
+	&dummy_remove,
 	&compress_deinit
 };
 
@@ -369,6 +370,7 @@ create_worker_threads(uint n)
 				   thd)) {
 			msg("compress: pthread_create() failed: "
 			    "errno = %d", errno);
+			pthread_mutex_unlock(&thd->ctrl_mutex);
 			goto err;
 		}
 	}
@@ -385,6 +387,14 @@ create_worker_threads(uint n)
 	return threads;
 
 err:
+	while (i > 0) {
+		comp_thread_ctxt_t *thd;
+		i--;
+		thd = threads + i;
+		pthread_mutex_unlock(&thd->ctrl_mutex);
+	}
+
+	my_free(threads);
 	return NULL;
 }
 
