@@ -52,7 +52,7 @@
 #include <netinet/ip.h>
 #include <netdb.h>
 #include <netinet/tcp.h>
-#define IS_SOCKET_EINTR(err) (err == SOCKET_EINTR)
+#define IS_SOCKET_EINTR(err) ((err) == SOCKET_EINTR)
 #else
 #include <ws2tcpip.h>
 #define O_NONBLOCK 1
@@ -84,6 +84,12 @@
 #define HAVE_SOCKET_EWOULDBLOCK 1
 #endif
 
+#ifdef _AIX
+#ifndef MSG_DONTWAIT
+#define MSG_DONTWAIT 0
+#endif
+#endif
+
 /* Function prototypes */
 my_bool pvio_socket_set_timeout(MARIADB_PVIO *pvio, enum enum_pvio_timeout type, int timeout);
 int pvio_socket_get_timeout(MARIADB_PVIO *pvio, enum enum_pvio_timeout type);
@@ -786,7 +792,13 @@ my_bool pvio_socket_connect(MARIADB_PVIO *pvio, MA_PVIO_CINFO *cinfo)
     else
 #endif
     {
-      strcpy(UNIXaddr.sun_path, cinfo->unix_socket);
+      size_t sun_path_size = sizeof(UNIXaddr.sun_path);
+      strncpy(UNIXaddr.sun_path, cinfo->unix_socket, sun_path_size - 1);
+      if (sun_path_size == strlen(UNIXaddr.sun_path) + 1 && UNIXaddr.sun_path[sun_path_size - 1] != '\0')
+      {
+        /* Making the string null-terminated */
+        UNIXaddr.sun_path[sun_path_size - 1] = '\0';
+      }
       port_length= sizeof(UNIXaddr);
     }
     if (pvio_socket_connect_sync_or_async(pvio, (struct sockaddr *) &UNIXaddr, port_length))
