@@ -19,9 +19,10 @@
 #pragma interface                      /* gcc class implementation */
 #endif
 
+#include "table.h"
+
 struct st_join_table;
 class handler;
-struct TABLE;
 class THD;
 class SQL_SELECT;
 class Copy_field;
@@ -53,24 +54,22 @@ struct READ_RECORD
   typedef int (*Setup_func)(struct st_join_table*);
 
   TABLE *table;                                 /* Head-form */
-  //handler *file;
-  TABLE **forms;                                /* head and ref forms */
   Unlock_row_func unlock_row;
-  Read_func read_record;
+  Read_func read_record_func;
   THD *thd;
   SQL_SELECT *select;
-  uint cache_records;
-  uint ref_length,struct_length,reclength,rec_cache_size,error_offset;
-  uint index;
+  uint ref_length, reclength, rec_cache_size, error_offset;
   uchar *ref_pos;				/* pointer to form->refpos */
-  uchar *record;
   uchar *rec_buf;                /* to read field values  after filesort */
   uchar	*cache,*cache_pos,*cache_end,*read_positions;
   struct st_sort_addon_field *addon_field;     /* Pointer to the fields info */
   struct st_io_cache *io_cache;
-  bool print_error, ignore_not_found_rows;
+  bool print_error;
   void    (*unpack)(struct st_sort_addon_field *, uchar *, uchar *);
 
+  int read_record() { return read_record_func(this); }
+  uchar *record() const { return table->record[0]; }
+
   /* 
     SJ-Materialization runtime may need to read fields from the materialized
     table and unpack them into original table fields:
