@@ -14,7 +14,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA */
 
 #include "sql_error.h"
 
@@ -115,7 +115,7 @@ class Ed_row;
   automatic type conversion.
 */
 
-class Ed_result_set: public Sql_alloc
+class Ed_result_set
 {
 public:
   operator List<Ed_row>&() { return *m_rows; }
@@ -129,7 +129,10 @@ public:
 
   size_t get_field_count() const { return m_column_count; }
 
+  static void *operator new(size_t size, MEM_ROOT *mem_root)
+  { return alloc_root(mem_root, size); }
   static void operator delete(void *ptr, size_t size) throw ();
+  static void operator delete(void *, MEM_ROOT *){}
 private:
   Ed_result_set(const Ed_result_set &);        /* not implemented */
   Ed_result_set &operator=(Ed_result_set &);   /* not implemented */
@@ -214,21 +217,6 @@ public:
   */
   bool execute_direct(Server_runnable *server_runnable);
 
-  /**
-    Get the number of result set fields.
-
-    This method is valid only if we have a result:
-    execute_direct() has been called. Otherwise
-    the returned value is undefined.
-
-    @sa Documentation for C API function
-    mysql_field_count()
-  */
-  ulong get_field_count() const
-  {
-    return m_current_rset ? m_current_rset->get_field_count() : 0;
-  }
-
   /**
     Get the number of affected (deleted, updated)
     rows for the current statement. Can be
