@@ -12,7 +12,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA */
 
 
 /**
@@ -29,10 +29,9 @@
   - "profiling_history_size", integer, session + global, "Num queries stored?"
 */
 
-#include <my_global.h>
+#include "mariadb.h"
 #include "sql_priv.h"
 #include "sql_profile.h"
-#include <my_sys.h>
 #include "sql_show.h"                     // schema_table_store_record
 #include "sql_class.h"                    // THD
 
@@ -120,8 +119,10 @@ int make_profile_table_for_show(THD *thd, ST_SCHEMA_TABLE *schema_table)
       continue;
 
     field_info= &schema_table->fields_info[i];
+    LEX_CSTRING field_name= {field_info->field_name,
+                             strlen(field_info->field_name) };
     Item_field *field= new (thd->mem_root) Item_field(thd, context,
-                                      NullS, NullS, field_info->field_name);
+                                      NullS, NullS, &field_name);
     if (field)
     {
       field->set_name(thd, field_info->old_name,
@@ -283,11 +284,10 @@ QUERY_PROFILE::~QUERY_PROFILE()
 /**
   @todo  Provide a way to include the full text, as in  SHOW PROCESSLIST.
 */
-void QUERY_PROFILE::set_query_source(char *query_source_arg,
-                                     uint query_length_arg)
+void QUERY_PROFILE::set_query_source(char *query_source_arg, size_t query_length_arg)
 {
   /* Truncate to avoid DoS attacks. */
-  uint length= MY_MIN(MAX_QUERY_LENGTH, query_length_arg);
+  size_t length= MY_MIN(MAX_QUERY_LENGTH, query_length_arg);
 
   DBUG_ASSERT(query_source == NULL); /* we don't leak memory */
   if (query_source_arg != NULL)
