@@ -5,9 +5,12 @@
 /***********************************************************************/
 #pragma once
 
-#ifndef __WIN__
+#if defined(_WIN32)
+static PCSZ slash = "\\";
+#else // !_WIN32
+static PCSZ slash = "/";
 #define stricmp strcasecmp
-#endif // !__WIN__
+#endif // !_WIN32
 
 typedef int(__stdcall* XGETREST) (char*, bool, PCSZ, PCSZ, PCSZ);
 
